#!/usr/bin/perl

# NagiosAuth.pm - The authentication backend for nagios grapher
# Copyright (C) 2004, NETWAYS GmbH, Gerd Mueller, Marius Hein
# $Id: NagiosGrapher.pm 610 2004-10-19 12:50:07Z gmueller $

#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# $Id: NagiosGrapher.pm 610 2004-10-19 12:50:07Z gmueller $

package NagiosGrapher;

use strict;
use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $VERSION);

use Exporter;
$VERSION = 1.00;              # Or higher
@ISA = qw(Exporter);

my @configuration=();
my $nagiosversion=1;

my $loaded_cfg=0;
my $nagios_loaded_cfg=0;

my %ngraphcfg;

my $ngraphcfg = "/etc/nagios/ngraph.cfg";
my $configfile=NagiosGrapher::get_ngraphercfg_value('nagios_config');

sub read_nagios_cfg {
	my @cfgfiles;
	my ($cfg) = @_;
	@configuration=();
	if(open(CFG,"<".$cfg)) {
		while(<CFG>) {
	 		if(m/^cfg_file/) {
	 			# Nagios V.1.X
	 			my ($dummy,$cfgfile)=split(/=/);
	 			push(@cfgfiles,$cfgfile);
	 		} elsif (m/^object_cache_file/) {
	 			# Nagios V.2.X and higher
	 			my ($dummy,$cfgfile)=split(/=/);
	 			@cfgfiles=();
	 			push(@cfgfiles,$cfgfile);
	 			$nagiosversion=2;
	 			last;
	 		}
		}
		close(CFG);
	}
	
	# read configurations
	foreach(@cfgfiles) {
		if(open(CFG,"<".$_)) {
			while(<CFG>) {
				push(@configuration,$_);
			}
			close(CFG);
		}
	}
	$nagios_loaded_cfg=1;
	return;
}

sub parse_nagios_cfg {
	my $rw="";
	my $act_item="";
	my ($type,$item,$line)=@_;
	
	return "" if($item eq "");
	
	my @items = split(/,/,$item);
	
	# parse all items
	foreach $item (@items) {
		$rw.="," if($rw ne "");
		foreach(@configuration) {
			# item defined?
			if(/define $type {/../\t}/) {
				my $search=$type."_name";
				# is this the item we are looking for?
				if(m/$search\s*(.*)/) {
					$act_item=$1;
				# return the line we want
				} elsif($act_item eq $item and m/\s*$line\s*(.*)/) {
					$rw.=$1;
					last;
				}
			}
		}
	}
	return $rw;
}


sub AuthCheck {
	my $rw=0;
	my ($cfg,$host,$user)=@_;

    read_nagios_cfg($cfg) if(!$nagios_loaded_cfg);
    
    if($nagiosversion==2) {
    	# Nagios Version 2.? Perform test
    	$rw=1 if(parse_nagios_cfg('contactgroup',parse_nagios_cfg('host',$host,'contact_groups'),'members') =~ m/$user(,|$)/);
    } else {
    	# Nagios Version 1.X isn't supported yet so everything is allowd 
    	$rw=1;
    }
	return $rw;
}

sub read_ngrapher_cfg {
	if(open(CFG,"<".$ngraphcfg)) {
		while(<CFG>) {
	 		if(m/\s*define\s+config\s*{/../\s*}/) {
	 			s/^\s*//;
	 			chomp();
	 			if(m/(\w*[_]*\w*)\s+(.*)$/) {
	 				my $key=$1;
	 				my $value=$2;
		 			
		 			$value.="/" if($key =~ m/path$/ && $value !~ m/\/$/);
		 			$ngraphcfg{$key}=$value;
		 			
	 			}
	 		}
		}
		$loaded_cfg=1;
		close(CFG);
	}
	return;
}

sub get_ngraphercfg_value {
	read_ngrapher_cfg() if(!$loaded_cfg);
	my ($key)=@_;
	my $rw=$ngraphcfg{$key};
	$rw=$ngraphcfg if($key eq "ngraphcfg");

	return $rw;
}


1;