#! /usr/bin/perl

# collect2_data.pl - The collector of nagios_grapher
# Copyright (C) 2004, NETWAYS GmbH, Gerd Mueller, Marius Hein
# $Id: collect2.pl 704 2004-11-25 12:07:57Z mhein $
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

use strict;
use RRDs;
use XML::Simple;
use POSIX;
use Data::Dumper;
use Digest::MD5 qw(md5_hex);
use Getopt::Long;
use File::Copy;
use File::Basename;
use NagiosGrapher;
use vars qw (
  $progname
  $rrdpath
  $rrdtool
  $configfile
  $url
  $step
  $host
  $service
  $perf
  $perfdata
  $orig_service
  @lines
  $block
  @blocks
  %tmp
  @tmp
  $path
  $file
  @system
  $update
  $template
  %values
  $key
  $rrd_err
  @srvext_lines
  $serviceextinfo
  $serviceext_path
  $serviceext_host
  $serviceext_type
  $check_host
  $check
  $srvext
  $xml_hash
  $xml_file
  $xml_tmp
  $xs
  $pipe
  $fifo_string
  $opt_verbose
  $opt_help
  $i
  $restart
  @sigs
  $image_tag
  $new_image_tag
  $graph_url
  $log_file
  $log_level
  $dummy
);

# LogLevel
my $LOG_VALUES_OK=1;
my $LOG_VALUES_NOK=2;
my $LOG_PRG_STATUS=4;
my $LOG_SERVICEEXT=8;
my $LOG_RRD_STATUS=16;
my $LOG_PIPE=32;

# signals
@sigs = keys(%SIG);
for (@sigs) {
    $SIG{$_} = \&sig_handler;
}

# subs
sub print_help;
sub sig_handler;
sub daemonize;

# Configvalues
$progname = basename($0);

$pipe            = NagiosGrapher::get_ngraphercfg_value('pipe');
$rrdpath         = NagiosGrapher::get_ngraphercfg_value('rrdpath');
$configfile      = NagiosGrapher::get_ngraphercfg_value('ngraphcfg');
$serviceext_type = NagiosGrapher::get_ngraphercfg_value('serviceext_type');
$serviceextinfo  = NagiosGrapher::get_ngraphercfg_value('serviceextinfo');
$serviceext_path = NagiosGrapher::get_ngraphercfg_value('serviceext_path');
$image_tag       = NagiosGrapher::get_ngraphercfg_value('icon_image_tag');

$log_file       = NagiosGrapher::get_ngraphercfg_value('log_file');
$log_level       = NagiosGrapher::get_ngraphercfg_value('log_level');

$url = NagiosGrapher::get_ngraphercfg_value('url');

$xml_file = $rrdpath . "index.xml";
$xml_tmp  = "/tmp/nagios_graph.xml.tmp";
$xs       = new XML::Simple(
    XMLDecl    => 1,
    RootName   => 'main',
    KeyAttr    => 'name',
    ForceArray => 1
);
$step    = "60";
$restart = 0;

# Fetching some options...
Getopt::Long::Configure('bundling');
GetOptions(
    "v"    => \$opt_verbose,
    "h"    => \$opt_help,
    "help" => \$opt_help,
);

unless ($opt_verbose) {
    &daemonize;
}

if($opt_verbose) {
	$opt_verbose=127;
} else {
	$opt_verbose=$log_level;
}

# Print help if wanted
print_help if ($opt_help);

print_log ("PRG: Starting up $progname...") if ($opt_verbose & $LOG_PRG_STATUS);

# Restart marker
RESTART:

# Setting Up Named Pipe
unless ( -p $pipe ) {
    unlink $pipe;
    POSIX::mkfifo( $pipe, 0666 )
      || die("Can't mknod $pipe: $!");
}

# Setting up main rrd path
unless ( -e $rrdpath ) {
    mkdir($rrdpath);
    print_log ("RRD: Creating path $rrdpath ...") if ($opt_verbose & $LOG_RRD_STATUS);
}

# Getting XML Data
if ( -e $xml_file ) {
    $xml_hash = $xs->XMLin($xml_file);
}

# Create an array with config lines...
open( CF, "<$configfile" )
  || die("Can't open configfile '$configfile'!");
@lines = <CF>;
close(CF);

# Parsing the Array...
$block = "FALSE";
undef(@blocks);
foreach (@lines) {
    unless ( $_ =~ "^#" || $_ =~ "^\n" ) {

        # Setting Blockcheck FALSE and add to values hash
        if ( $_ =~ "\}" && $block eq 'TRUE' ) {
            $block = 'FALSE';
            $tmp{graph_value} =~ s/\s/_/g;
            push( @blocks, {%tmp} ) unless ( $tmp{type} =~ /cdef|gprint/i );
            %tmp = ();
        }

        # Fill the temp hash with the config values
        if ( $block eq 'TRUE' ) {
            $_ =~ s/^\s*|\n//gi;

            #@tmp = split(/\s+/,$_,2);
            if ( $_ =~ /^(.*?)\s+(.*?)$/i ) {
                $tmp{$1} = $2;
            }

            # $tmp{$tmp[0]}=$tmp[1];
        }

        # Setting Blockcheck TRUE
        $block = "TRUE" if ( $_ =~ "define ngraph{\n" );
    }
}

# print Dumper(\@blocks);

# LOOP MAIN
while (1) {

    if ($restart) {
        print_log ("PRG: Restarting $progname ...") if ($opt_verbose & $LOG_PRG_STATUS);
        $restart = 0;
        goto RESTART;
    }

    open( FIFO, $pipe )
      || die "Can't open pipe ($pipe): $!";
    $fifo_string = <FIFO>;
    close(FIFO);

    print_log ("PIPE: $fifo_string") if ( defined($fifo_string) && ($opt_verbose & $LOG_PIPE));

    # Splitting pipe values
    ( $host, $service, $perf, $perfdata ) = split( /\t/, $fifo_string );

    if (   defined($host)
        && defined($service)
        && defined($perf)
        && defined($fifo_string) )
    {

		my $log_header="[".$host."][".$service."]:";
        # Change defaults...
        $orig_service = $service;
        $perf =~ tr/[A-Z]/[a-z]/;
        $perf =~ s/\\/\\/g;

        if ( @blocks > 0 ) {

            # Getting the values from RegEX and perf data
            $check = "FALSE";
            foreach (@blocks) {
                $_->{service_name} =~ s/\s+$//g;
                if ( $service =~ /$_->{service_name}/ ) {
                    if ( defined($perfdata)
                        && exists( $_->{graph_perf_regex} ) )
                    {

                        # Taking PerfData
                        if ( $perfdata =~ m/$_->{graph_perf_regex}/i ) {
                            $values{ $_->{graph_value} } = $1;
                            $check = "TRUE";
                        }
                        else {
                            print_log ("VALUES: ".$log_header."No matching perfdata values found...")
                              if ($opt_verbose & $LOG_VALUES_NOK);
                            goto OUT;
                        }
                    }
                    else {

                        # Taking Plugin Output
                        if ( $perf =~ m/$_->{graph_log_regex}/i ) {
                            $values{ $_->{graph_value} } = $1;
                            $check = "TRUE";
                        }
                        else {
                            print_log ("VALUES: ".$log_header."No matching output values found...")
                              if ($opt_verbose & $LOG_VALUES_NOK);
                            goto OUT;
                        }
                    }
                }
            }

            # If no regex match the service ... next round
            if ( $check eq "FALSE" ) {
                print_log ("VALUES: ".$log_header."No Action taken...") if ($opt_verbose & $LOG_VALUES_NOK);
                goto OUT;
            }

            # Checking XML Entry
            if ( $xml_hash->{host}->{$host}->{service}->{$service}
                ->{service_name} )
            {

                # print "Exists\n";
            }
            else {

                # new default values
                $xml_hash->{host}->{$host}->{service}->{$service} = {
                    rrd_file =>
                      md5_hex( $host . "_" . $service . "_" . $orig_service )
                      . ".rrd",
                    service_title => $orig_service,
                    service_name  => $service,
                };
                unlink($xml_tmp) if ( -e $xml_tmp );
                open( SF, ">$xml_tmp" );
                print SF $xs->XMLout($xml_hash);
                close(SF);
                unlink($xml_file) if ( -e $xml_file );
                move( $xml_tmp, $xml_file );
            }

            # Clearing some var's
            undef @system;
            undef $update;
            undef $template;
            undef $rrd_err;

            # Adding path if not exists
            $path = $rrdpath . $host;
            mkdir($path) if ( !( -e $path ) );

            # Setting up the RRD Filename
            $file = $file =
                $path . "/"
              . $xml_hash->{host}->{$host}->{service}->{$service}->{rrd_file};

            # If the RRD not exists:
            if ( !( -e $file ) ) {

                # Adding Graph Value Key's
                foreach (@blocks) {
                    if ( $service =~ /$_->{service_name}/ ) {

                        # change the default rrdtype if a different is defined
                        my $rrd_type = "GAUGE";
                        $rrd_type = $_->{rrd_type}
                          if ( exists( $_->{rrd_type} ) );
                        push @system,
                          "DS:"
                          . $_->{graph_value} . ":"
                          . $rrd_type
                          . ":600:U:U";
                    }
                }

                # Adding some default average frames
                push @system, "RRA:AVERAGE:0.5:5:600";
                push @system, "RRA:AVERAGE:0.5:30:600";
                push @system, "RRA:AVERAGE:0.5:120:600";
                push @system, "RRA:MAX:0.5:120:600";
                push @system, "RRA:AVERAGE:0.5:1440:600";
                push @system, "RRA:MAX:0.5:1440:600";

                # Create the RRD
                RRDs::create( $file, "--step", $step, @system );
                
                if($opt_verbose & $LOG_RRD_STATUS) {
                	$dummy="";
                	foreach (@system) {
                		$dummy.=$_;
                	}
                	print_log("RRD: rrdtool create $file --step=$step $dummy");
                }
                
                $rrd_err = RRDs::error;

                # Debugging for create RRD's
                if ($rrd_err) {
                    print_log ("RRD: ".$log_header."$rrd_err") if ($opt_verbose & $LOG_RRD_STATUS);
                }
                else {
                    print_log ("RRD: ".$log_header."$file - successfully created!") if ($opt_verbose & $LOG_RRD_STATUS);
                }
            }

            # Check if an serviceext entry exists
            if ( $serviceext_type eq "MULTIPLE" ) {
                mkdir $serviceext_path unless ( -e $serviceext_path );
                $serviceext_host = $host;
                $serviceext_host =~ s/\W/_/gs;
                $serviceextinfo =
                  $serviceext_path . "/" . $serviceext_host . ".cfg";
            }
            elsif ( $serviceext_type eq "SINGLE" && !( -e $serviceextinfo ) ) {
                die("No serviceextinfo file defined, can't write!");
            }
            system( "touch", $serviceextinfo ) if ( !-e $serviceextinfo );

            open( SRVEXT, "<$serviceextinfo" )
              || die("Can open '$serviceextinfo' for reading\n");
            @srvext_lines = <SRVEXT>;
            close(SRVEXT);

            $check_host = $host;
            $check_host =~ s/\./\./g;

            $check = "0";
            foreach (@srvext_lines) {
                if($_ =~ /define\s+serviceextinfo\s*\{/../\}/) {
                    # print "i: $_\n";
                    $check = "1" if ( $_ eq "\thost_name\t\t$host\n" && $check eq "0" );
                    
                    if ( $_ eq "\tservice_description\t$orig_service\n" && $check eq "1" ) {
                        $check = "2";
                        last;
                    }
                    # print "c: $check\n";
                } else {
                    $check = "0";
                }
            }

            unless ( $check eq "2" ) {
                print "Creating New !!!\n\n";
                print_log ("SERVICEEXT: ".$log_header."Entry in serviceextinfo not exists, create NEW!")
                  if ($opt_verbose & $LOG_SERVICEEXT);

                # Writing the serviceextinfo for the NAGIOS FE
                $new_image_tag = $image_tag;
                $graph_url     = "host=$host&service=$service&page=";

                # Adding multiple page support if needed.
                $i = 0;
                foreach (@blocks) {
                    if ( $service =~ /$_->{service_name}/
                        && exists( $_->{page} ) )
                    {
                        if ( $i <= 0 ) {
                            $graph_url .= $_->{page};
                        }
                        else {
                            $graph_url .= ',' . $_->{page};
                        }
                        $i++;
                    }
                }

                $new_image_tag =~ s/###HOST###/$host/gi;
                $new_image_tag =~ s/###SERVICE###/$service/gi;
                $new_image_tag =~ s/###URL###/$graph_url/gi;

                open( OUT, ">>$serviceextinfo" )
                  || die "Can not open file '$serviceextinfo'\n";
                print OUT "\n# ExtInfo for $host, $service\n";
                print OUT "define serviceextinfo{\n";
                print OUT "\thost_name\t\t$host\n";
                print OUT "\ticon_image\t\t$new_image_tag\n";
                print OUT "\tservice_description\t$orig_service\n";
                print OUT "\tnotes_url\t\t$url?$graph_url\n";
                print OUT "}\n\n";
                close(OUT);
            }

            # Updating the RRD_FILE...
            $dummy="";
            foreach $key ( keys(%values) ) {
                $update = "N" if ( !defined($update) );
                $update .= ":" . $values{$key};
                $template.= ":" if(defined($template));
                $template.=$key;
                $dummy.=$key."=".$values{$key}." ";
            }
            print_log ("VALUES: ".$log_header." ".$dummy) if($opt_verbose & $LOG_VALUES_OK);

            # Updating... really
            RRDs::update( $file, "--template",$template, $update );
            print_log("RRD: rrdtool update $file --template=$template $update") if($opt_verbose & $LOG_RRD_STATUS);
            
            $rrd_err = RRDs::error;

            # Debugging for update RRD's
            if ($rrd_err) {
                print_log ("RRD: ".$log_header."$rrd_err") if ($opt_verbose & $LOG_RRD_STATUS);
            }
        }

    }

  OUT:

    # Clearing another values...
    %values = ();
#    print_log "\n" if ($opt_verbose);
}

# End LOOP MAIN

# Default exit
exit(0);
1;

# Subs:
# Helpscreen
sub print_help {
    print "$progname: Helpscreen\n";
    print "\t-v\t\tverbose output\n";
    print "\t-h\t\thelp screen\n";
    print "\n";
    exit(0);
}

# Daemonize the Script...
sub daemonize {
    chdir '/' or die "Can't chdir to /: $!";
    open STDIN,  '/dev/null'   or die "Can't read /dev/null: $!";
    open STDOUT, '>>/dev/null' or die "Can't write to /dev/null: $!";
    open STDERR, '>>/dev/null' or die "Can't write to /dev/null: $!";
    defined( my $pid = fork ) or die "Can't fork: $!";
    exit if $pid;
    setsid or die "Can't start a new session: $!";
    umask 0;
}

# Signalhandler
sub sig_handler {
    my $signal = shift;
    %SIG = ();
    if ( $signal eq 'TERM' ) {
        print_log ("PRG: $progname Terminated ...") if ($opt_verbose & $LOG_PRG_STATUS);
        close(LOG);
        unlink($pipe)
          || die("Can't unlink $pipe: $!");
        exit(0);
    }
    elsif ( $signal eq 'HUP' ) {
        print_log ("PRG: $progname reload Configuration next loop ...")
          if ($opt_verbose & $LOG_PRG_STATUS);
        $restart = 1;
        $SIG{$signal} = \&sig_handler;
    }
    elsif ( $signal eq 'INT' ) {
        print_log ("PRG: $progname interupted ..") if ($opt_verbose & $LOG_PRG_STATUS);
        unlink($pipe)
          || die("Can't unlink $pipe: $!");
        exit(0);
    }
    elsif ( $signal ne "__WARN__" ) {
        print_log ("$progname: Got signal: $signal ...") if ($opt_verbose & $LOG_PRG_STATUS);
        $SIG{$signal} = \&sig_handler;
    }
}

sub print_log {
	my ($out) = @_;
	chomp($out);
	open (LOG,">>".$log_file) || die "Can't open logfile!\n";
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
	printf (LOG "%02d-%02d-%02d %02d:%02d:%02d %s\n",$year+1900,$mon+1,$mday,$hour,$min,$sec,$out);
	close (LOG);
}