#!/usr/bin/perl -w

# graphs.cgi - The Frontend of the nagios grapher
# Copyright (C) 2004, NETWAYS GmbH, Gerd Mueller, Marius Hein
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# Main
use strict;
use Data::Dumper;
use CGI qw/:standard /;
use CGI::Carp qw(fatalsToBrowser);
use NagiosGrapher;
use vars qw (
    $self
    $stylesheet
    $q
    %params
    $height
    $width
    $graph_script
    $host
    $service
    @rrd
    %rrd_desc
    %rrd_cmd
    $title
    $refresh
    %refresh_values
    $page
    @page_array
    %page_hash
    $page_act
    $img_source
);

# Configuration
NagiosGrapher::read_ngrapher_cfg();

# The SUBS:
sub make_select_options;

$graph_script = "./rrd2-graph.cgi";

$q = new CGI;
%params = $q->Vars;
%refresh_values = (
                    ""     => "None",
                    "30"    => "30 seconds",
                    "60"    => "1 minute",
                    "300"   => "5 minutes",
                    "600"   => "10 minutes",
                  );

$refresh=$params{refresh};
$refresh=300 if (!defined($refresh));

$height=$params{height};
$height=250 if (!defined($height) || length($height) <= 0);

$width=$params{width};
$width=595 if (!defined($width) || length($width) <= 0);

$host=$params{host};
$service=$params{service};

$page=$params{page};
@page_array = split(/,/,$page) if (length($page) > 0);
foreach (@page_array) { $page_hash{$_}=$_; }
$page_act=$params{act_page};
$page_act=$page_array[0] if (!defined($page_act) || length($page_act) <= 0);


# RRD Type Array
@rrd = (0..4);
@rrd = sort(@rrd);
# RRD Description HASH
%rrd_desc = (0 => "Actual", 1 => "Daily", 2 => "Weekly", 3 => "Monthly", 4 => "Yearly");
# RRD TimeInterval HASH
%rrd_cmd = (0 => 5400, 1 => 129600, 2 => 907200, 3 => 4017600, 4 => 48211200);
# SiteTitle
$title = "graph analysis for $host, $service";

# Default Stylesheet
$stylesheet =<<ECSS;
body { font-family: 'Verdana', 'Arial'; font-size: 8pt; color: #666666; background-color: #f1f1f1; }
td { font-size: 8pt; color: #333333; }
a { font-size: 8pt; text-decoration: none; color: #ff8000; font-weight: bolder; }
a:hover { text-decoration: underline; }
.head2 { color: #333333; font-size: 10pt; font-weight: normal; }
.head2 em { color: #cc0000; font-size: 10pt; font-weight: normal; }
.head1 { color: #333333; font-size: 12pt; font-weight: bolder; }
.table1 { border: 1px #333333 solid; }
div.line { height: 1px; font-size: 1px; background-color: #000080; margin: 5px 0px 5px 0px; }
ECSS

# Starting the HTML Output ...
$self = $q->self_url;
$self =~ s/(;)/\&/gi; # making selfreferencing URL without ';' for meta reload tag.
$q->default_dtd('-//W3C//DTD HTML 4.01 Transitional//EN');
print $q->header(-type=>'text/html', -expires=>'+60s');
if ($refresh) {
print $q->start_html(
                        -lang=>'de-de',
                        -title=>$title,
                        -style=>{
                                 -code=>$stylesheet
                                },
                        -head=>meta({
                                -http_equiv => "refresh",
                                -content => $refresh. "; URL=". $self,
                               })
                    );
} else {
   print $q->start_html(
                        -lang=>'de-de',
                        -title=>$title,
                        -style=>{
                                 -code=>$stylesheet
                                },
                    );
}


#print "<pre>";
#print $page_act;
#print "</pre>";

print "<table border=\"1\" cellpadding=\"4\" cellspacing=\"0\" class=\"table1\" style=\"border-collapse: collapse;\" align=\"center\">\n";
print '<tr><td colspan="2">'. "\n";

# The MENU:
print $q->a({-name=>"TOP"}, $q->span({-class => "head1"}, "NETWAYS Nagios Grapher"));
print "<div class=\"line\"></div>";
print $q->span({-class => "head2"}, "Graphs for <em>$host</em>, <em>$service</em>:"). "&nbsp;";
foreach (@rrd) {
    print "[&nbsp;",
        $q->a({href=>$q->self_url. "#". $rrd_desc{$_}}, $rrd_desc{$_}),
        "&nbsp;]&nbsp;",
        "\n";
}
print $q->br, $q->br,"\n\n",
    $q->start_form(-method=>"get", -action=>$q->self_url, -enctype=>"application/x-www-form-urlencoded"),
    $q->hidden(-name=>"service", -value=>$service),
    $q->hidden(-name=>"host", -value=>$host),
    $q->hidden(-name=>"page", -value=>$page),
    "width:&nbsp;",
    $q->textfield(-name=>"width", -default=>$width, -override=>1, -size=>4),
    "&nbsp;height:&nbsp;",
    $q->textfield(-name=>"height", -default=>$height, -override=>1, -size=>4),
    "&nbsp;",
    "Refresh:&nbsp;",
    $q->popup_menu(
                    -name=>"refresh",
                    -default=>$refresh,
                    -values=>[keys(%refresh_values)],
                    -labels=>\%refresh_values
                      );

if ($page) {
print "&nbsp;Page:&nbsp;", $q->popup_menu(
                     -name=>"act_page",
                     -default=>$page_act,
                     -values=>[keys(%page_hash)],
                     -labels=>\%page_hash
                     );
}



print "&nbsp;", $q->submit(-name=>"cmd_change", -value=>"Change"), 
    $q->endform,
    "[&nbsp;", $q->a({-href=>"#", -onclick=>"window.location.reload(); return true;"}, "Reload"), "&nbsp;]",
    "&nbsp;[&nbsp;", $q->a({-href=>"#", -onclick=>"top.close(); return true;"}, "Close"), "&nbsp;]",
    $q->br, $q->br,
    "\n\n";
    
print '</td></tr>'. "\n";
foreach (@rrd) {
    print "<tr><td>". $rrd_desc{$_}. " graph:",
        $q->br,
        "[&nbsp;", $q->a({-href=>$q->self_url. "#TOP"}, "To top"), "&nbsp;]",
        "</td><td>\n";
    $img_source = $graph_script. '?host='. $host. '&service='. $service. '&start=-'. $rrd_cmd{$_}. '&title='. $rrd_desc{$_}. '&width='. $width. '&height='. $height. '&act_page='. $page_act;
    print $q->a({name=>$rrd_desc{$_}},
                $q->a({-href => $img_source, -target=>"_blank"},
                $q->img({src => $img_source, border => 0, alt => $rrd_desc{$_}}))
               );
    print "</td></tr>\n";
}

print "</table>\n";
# End HTML Output ...
print $q->end_html();
exit(0);
1;

# The Subs:
sub make_select_options {
    my (%values);
    %values = %{ $_[0] };
}