#! /usr/bin/perl -w

# rrd2-graph.cgi - Returns an Image from an RRDDB using the ngraph configfile
# Copyright (C) 2004, NETWAYS GmbH, Gerd Mueller, Marius Hein
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

use strict;
use CGI qw(:standard);
use IO::Handle;
use RRDs;
use XML::Simple;
#use Data::Dumper;
use NagiosGrapher;
use GD;
use Time::Local;

use vars qw (
    $path
    $rrdtool
    $configfile
    $height
    $width
    $host
    $service
    $title
    $end
    $start
    $block
    @lines
    @values
    @graphs
    @prints
    %tmp
    @blocks
    @tmp
    $ERR
    $x_scale
    $page_act
    $xml_hash
    $xml_file
    $xs
    $filename
    $nagios_config
    @options
);

# Configuration
NagiosGrapher::read_ngrapher_cfg();
$path=NagiosGrapher::get_ngraphercfg_value('rrdpath');;
$xml_file = $path. "index.xml";

$configfile=NagiosGrapher::get_ngraphercfg_value('ngraphcfg');
$nagios_config=NagiosGrapher::get_ngraphercfg_value('nagios_config');

$xs = new XML::Simple(
                      XMLDecl => 1,
                      RootName => 'main',
                      KeyAttr => 'name',
                      ForceArray => 1
                      );

# CGI Query Params
$host=param("host");
$service=param("service");
$title=param("title");
$end=param("end");
$start=param("start");
$width=param("width");
$height=param("height");
$page_act=param("act_page");
undef $page_act if (length($page_act) <= 0);

if(!NagiosGrapher::AuthCheck($nagios_config,$host,$ENV{'REMOTE_USER'})) {
	print STDOUT "Content-type: image/png\n";
	print STDOUT "Content-disposition: filename=accessdenied.png\n\n";
	print_error("Sorry you are not allowed to view this graph!\nYour Nagios System");
	exit 0;
}

# Setting up default values...
$start="--start=".$start if(!defined($start) || $start ne "");
$end="--end=".$end if(!defined($end) || $end ne "");
$height=255 if (!defined($height) || $height eq "");
$width=595 if (!defined($width) || $width eq "");

@options=('-aPNG');

# Create an array with config lines...
open (CF, "<$configfile")
    ||die ("Can't open configfile '$configfile'!");
@lines = <CF>;
close (CF);

# Parsing the Array...
$block = "FALSE";
foreach (@lines) {
    unless($_ =~ "^#" || $_ =~ "^\n") {
        # Setting Blockcheck FALSE and add to values hash if
        # the service match the CGI Param
        if ($_ =~ "\}") {
            $block = 'FALSE';
            $tmp{graph_value} =~ s/\s/_/g;
            $tmp{graph_source} =~ s/\s/_/g;
            push(@blocks, {%tmp}) if ($service =~ /$tmp{service_name}/ );
            %tmp = {};
        }
        # Fill the temp hash with the config values
        if ($block eq 'TRUE') {
            $_ =~ s/^\s*|\n//gi;
            # @tmp = split(/\t+/);
            #@tmp = split(/\s+/,$_,2);
            #$tmp{$tmp[0]}=$tmp[1];
            if ($_ =~ /^(.*?)\s+(.*?)$/i) {
                $tmp{$1}=$2;
            }
        }
        # Setting Blockcheck TRUE
        $block = "TRUE" if ($_ =~ "define ngraph{\n");
    }
}

# Reading WDDX XML from index file...
if (-e $xml_file) {
    $xml_hash = $xs->XMLin($xml_file);
}

# Setting up some date as comment.
my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday)=localtime(time());
my $date=sprintf("[%02d:%02d/%02d.%02d.%04d]",$hour,$min,$mday,$mon,$year+1900);

# fetch the data...
if (@blocks > 0) {
    foreach (@blocks) {
        # Paging
        next if ($page_act && !($_->{page} eq $page_act));
        
        # Adding real ds
        push (@values, "DEF:". %$_->{graph_value}. "=$path". "$host/". $xml_hash->{host}->{$host}->{service}->{$service}->{rrd_file}. ":". %$_->{graph_value}. ":AVERAGE ") unless(%$_->{type} =~ /cdef|gprint|hrule/i);
        push (@graphs, %$_->{rrd_plottype}. ":". %$_->{graph_value}. "#". %$_->{rrd_color}. ":". %$_->{graph_legend}. "\\J") unless ( (exists(%$_->{hide}) && %$_->{hide} =~ /yes/i) || %$_->{type} =~ /cdef|gprint|hrule/i);
        
        # Adding CDEF's
        if (%$_->{type} =~ /cdef/i) {
            push(@values, "CDEF:". %$_->{graph_value}. "=". %$_->{graph_calc});
            push(@graphs, %$_->{rrd_plottype}. ":". %$_->{graph_value}. "#". %$_->{rrd_color}. ":". %$_->{graph_legend}. "\\J") unless ( (exists(%$_->{hide}) && %$_->{hide} =~ /yes/i) );
        }
        
        # Adding GPRINTs
        if (%$_->{type} =~ /gprint/i) {
            push(@prints, "GPRINT:". %$_->{print_source}. ":". uc(%$_->{print_function}). ":". %$_->{print_description}. "\\: ". %$_->{print_format}. "\\n");
        }
        
        # Adding HRULEs
        if (%$_->{type} =~ /hrule/i) {
            push(@graphs, "HRULE:". %$_->{hrule_value}. "#". %$_->{rrd_color});
        }
        
        unless (%$_->{type} =~ /gprint/i) {
            $x_scale.=" / " if(defined($x_scale) && $x_scale ne "" && $x_scale ne %$_->{graph_units} && (%$_->{graph_units} ne ""));
            $x_scale.=%$_->{graph_units} if ($x_scale ne %$_->{graph_units});
        }
    }
}

#VLINES 
if($start eq "--start=-5400") {
	# mark hours
	push(@graphs, "VRULE:".timelocal(0, 0, $hour, $mday, $mon, $year)."#FF0000");
	($sec,$min,$hour,$mday,$mon,$year,$wday,$yday)=localtime(time()-3600);
	push(@graphs, "VRULE:".timelocal(0, 0, $hour, $mday, $mon, $year)."#FF0000");
	
} elsif ($start eq "--start=-129600") {
	# mark days
	push(@graphs, "VRULE:".timelocal(0, 0, 0, $mday, $mon, $year)."#FF0000");
	($sec,$min,$hour,$mday,$mon,$year,$wday,$yday)=localtime(time()-86400);
	push(@graphs, "VRULE:".timelocal(0, 0, 0, $mday, $mon, $year)."#FF0000");
	
} elsif ($start eq "--start=-907200") {
	# mark weeks
	my $monday_seconds=($wday-1)*24*60*60;
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday)=localtime(time()-$monday_seconds);
	push(@graphs, "VRULE:".timelocal(0, 0, 0, $mday, $mon, $year)."#FF0000");
	
	($sec,$min,$hour,$mday,$mon,$year,$wday,$yday)=localtime(time()-604800);
	$monday_seconds=($wday-1)*24*60*60;
	my $monday=(localtime(time()-$monday_seconds-604800))[3];
	push(@graphs, "VRULE:".timelocal(0, 0, 0, $monday, $mon, $year)."#FF0000");
	
} elsif ($start eq "--start=-4017600") {
	# mark month
	push(@graphs, "VRULE:".timelocal(0, 0, 0, 1, $mon, $year)."#FF0000");
	($sec,$min,$hour,$mday,$mon,$year,$wday,$yday)=localtime(time()-2419200);
	push(@graphs, "VRULE:".timelocal(0, 0, 0, 1, $mon, $year)."#FF0000");
	
	# calendarweek workaround
	push(@options,'--x-grid=DAY:1:WEEK:1:WEEK:1:604800:Week %V');
	
} elsif ($start eq "--start=-48211200") {
	# mark year
	push(@graphs, "VRULE:".timelocal(0, 0, 0, 1, 0, $year)."#FF0000");
	($sec,$min,$hour,$mday,$mon,$year,$wday,$yday)=localtime(time()-29030400);
	push(@graphs, "VRULE:".timelocal(0, 0, 0, 1, 0, $year)."#FF0000");
}

# Make the output ...
STDOUT->autoflush(1);
print STDOUT "Content-type: image/png\n";
$filename = $xml_hash->{host}->{$host}->{service}->{$service}->{rrd_file};
$filename =~ s/\.rrd//i;
print STDOUT "Content-disposition: filename=". $filename. "_". $title. ".png\n\n";


RRDs::graph("-", @options, "-h$height", "-w$width","$start", "--title=$host(". $xml_hash->{host}->{$host}->{service}->{$service}->{service_title}. ") - $title Graph", "-v $x_scale", @values, @graphs, @prints, "COMMENT: NETWAYS GmbH - $date\\r");
$ERR = RRDs::error;
if ($ERR) {
	my $msg;
    $msg.= "Some error occured:\n";
    $msg.= "~~~~~~~~~~~~~~~~~~~\n\n";
    $msg.= "RRD Error: $ERR\n"; 
    $msg.= "Values:\n";
    foreach (@values) {
        $msg.= "  $_\n";
    }
    $msg.= "\nGraphs:\n";
    foreach (@graphs) {
        $msg.= "  $_\n";
    }
    $msg.= "\nGPrints:\n";
    foreach (@prints) {
        $msg.= "  $_\n";
    }
    $msg.= "\n$date\n";
    print_error($msg);
}
# Default exit and return true
exit (0);

sub print_error {
	my ($msg) = @_;
	my @lines = split(/\n/,$msg);
 	my $im = new GD::Image($width,$height);

  	# allocate some colors
    my $white = $im->colorAllocate(255,255,255);
    my $black = $im->colorAllocate(0,0,0);       
    my $red = $im->colorAllocate(255,0,0);      

    # make the background transparent and interlaced
    $im->transparent($white);
    $im->interlaced('true');
    my $line=0;
    foreach(@lines) {
		$im->string(gdSmallFont,0,$line,$_,$red);
    	$line+=12;
    }	
	  
	binmode STDOUT;
    print $im->png;	
}

1;